@@ -1,4 +1,4 @@
-/*	$OpenBSD: filemode.c,v 1.38 2024/02/22 12:49:42 job Exp $ */
+/*	$OpenBSD: filemode.c,v 1.47 2024/06/17 18:54:36 tb Exp $ */
 /*
  * Copyright (c) 2019 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -41,14 +41,56 @@
 #include "extern.h"
 #include "json.h"
 
-extern int		 verbose;
-
 static X509_STORE_CTX	*ctx;
 static struct auth_tree	 auths = RB_INITIALIZER(&auths);
 static struct crl_tree	 crlt = RB_INITIALIZER(&crlt);
 
 struct tal		*talobj[TALSZ_MAX];
 
+struct uripath {
+	RB_ENTRY(uripath)	 entry;
+	const char		*uri;
+	struct cert		*cert;
+};
+
+static RB_HEAD(uripath_tree, uripath) uritree;
+
+static inline int
+uripathcmp(const struct uripath *a, const struct uripath *b)
+{
+	return strcmp(a->uri, b->uri);
+}
+
+RB_PROTOTYPE(uripath_tree, uripath, entry, uripathcmp);
+
+static void
+uripath_add(const char *uri, struct cert *cert)
+{
+	struct uripath *up;
+
+	if ((up = calloc(1, sizeof(*up))) == NULL)
+		err(1, NULL);
+	if ((up->uri = strdup(uri)) == NULL)
+		err(1, NULL);
+	up->cert = cert;
+	if (RB_INSERT(uripath_tree, &uritree, up) != NULL)
+		errx(1, "corrupt AIA lookup tree");
+}
+
+static struct cert *
+uripath_lookup(const char *uri)
+{
+	struct uripath needle = { .uri = uri };
+	struct uripath *up;
+
+	up = RB_FIND(uripath_tree, &uritree, &needle);
+	if (up == NULL)
+		return NULL;
+	return up->cert;
+}
+
+RB_GENERATE(uripath_tree, uripath, entry, uripathcmp);
+
 /*
  * Use the X509 CRL Distribution Points to locate the CRL needed for
  * verification.
@@ -62,11 +104,11 @@ parse_load_crl(char *uri)
 
 	if (uri == NULL)
 		return;
-	if (strncmp(uri, "rsync://", strlen("rsync://")) != 0) {
+	if (strncmp(uri, RSYNC_PROTO, RSYNC_PROTO_LEN) != 0) {
 		warnx("bad CRL distribution point URI %s", uri);
 		return;
 	}
-	uri += strlen("rsync://");
+	uri += RSYNC_PROTO_LEN;
 
 	f = load_file(uri, &flen);
 	if (f == NULL) {
@@ -97,11 +139,11 @@ parse_load_cert(char *uri)
 	if (uri == NULL)
 		return NULL;
 
-	if (strncmp(uri, "rsync://", strlen("rsync://")) != 0) {
+	if (strncmp(uri, RSYNC_PROTO, RSYNC_PROTO_LEN) != 0) {
 		warnx("bad authority information access URI %s", uri);
 		return NULL;
 	}
-	uri += strlen("rsync://");
+	uri += RSYNC_PROTO_LEN;
 
 	f = load_file(uri, &flen);
 	if (f == NULL) {
@@ -115,7 +157,8 @@ parse_load_cert(char *uri)
 	if (cert == NULL)
 		goto done;
 	if (cert->purpose != CERT_PURPOSE_CA) {
-		warnx("AIA reference to bgpsec cert %s", uri);
+		warnx("AIA reference to %s in %s",
+		    purpose2str(cert->purpose), uri);
 		goto done;
 	}
 	/* try to load the CRL of this cert */
@@ -134,7 +177,7 @@ parse_load_cert(char *uri)
  * tree. Once the TA is located in the chain the chain is validated in
  * reverse order.
  */
-static void
+static struct auth *
 parse_load_certchain(char *uri)
 {
 	struct cert *stack[MAX_CERT_DEPTH] = { 0 };
@@ -146,18 +189,20 @@ parse_load_certchain(char *uri)
 	int i;
 
 	for (i = 0; i < MAX_CERT_DEPTH; i++) {
+		if ((cert = uripath_lookup(uri)) != NULL) {
+			a = auth_find(&auths, cert->certid);
+			if (a == NULL) {
+				warnx("failed to find issuer for %s", uri);
+				goto fail;
+			}
+			break;
+		}
 		filestack[i] = uri;
 		stack[i] = cert = parse_load_cert(uri);
 		if (cert == NULL || cert->purpose != CERT_PURPOSE_CA) {
-			warnx("failed to build authority chain");
+			warnx("failed to build authority chain: %s", uri);
 			goto fail;
 		}
-		if (auth_find(&auths, cert->ski) != NULL) {
-			assert(i == 0);
-			goto fail;
-		}
-		if ((a = auth_find(&auths, cert->aki)) != NULL)
-			break;	/* found chain to TA */
 		uri = cert->aia;
 	}
 
@@ -168,9 +213,9 @@ parse_load_certchain(char *uri)
 	}
 
 	/* TA found play back the stack and add all certs */
-	for (; i >= 0; i--) {
-		cert = stack[i];
-		uri = filestack[i];
+	for (; i > 0; i--) {
+		cert = stack[i - 1];
+		uri = filestack[i - 1];
 
 		crl = crl_get(&crlt, a);
 		if (!valid_x509(uri, ctx, cert->x509, a, crl, &errstr) ||
@@ -180,14 +225,16 @@ parse_load_certchain(char *uri)
 			goto fail;
 		}
 		cert->talid = a->cert->talid;
-		a = auth_insert(&auths, cert, a);
-		stack[i] = NULL;
+		a = auth_insert(uri, &auths, cert, a);
+		uripath_add(uri, cert);
+		stack[i - 1] = NULL;
 	}
 
-	return;
+	return a;
 fail:
 	for (i = 0; i < MAX_CERT_DEPTH; i++)
 		cert_free(stack[i]);
+	return NULL;
 }
 
 static void
@@ -197,7 +244,7 @@ parse_load_ta(struct tal *tal)
 	struct cert *cert;
 	unsigned char *f = NULL;
 	char *file;
-	size_t flen;
+	size_t flen, i;
 
 	/* does not matter which URI, all end with same filename */
 	filename = strrchr(tal->uri[0], '/');
@@ -219,11 +266,14 @@ parse_load_ta(struct tal *tal)
 		goto out;
 
 	cert->talid = tal->id;
+	auth_insert(file, &auths, cert, NULL);
+	for (i = 0; i < tal->urisz; i++) {
+		if (strncasecmp(tal->uri[i], RSYNC_PROTO, RSYNC_PROTO_LEN) != 0)
+			continue;
+		/* Add all rsync uri since any of them could be used as AIA. */
+		uripath_add(tal->uri[i], cert);
+	}
 
-	if (!valid_ta(file, &auths, cert))
-		cert_free(cert);
-	else
-		auth_insert(&auths, cert, NULL);
 out:
 	free(file);
 	free(f);
@@ -268,13 +318,13 @@ print_signature_path(const char *crl, const char *aia, const struct auth *a)
 	if (aia != NULL)
 		printf("                          %s\n", aia);
 
-	for (; a != NULL; a = a->parent) {
+	for (; a != NULL; a = a->issuer) {
 		if (a->cert->crl != NULL)
 			printf("                          %s\n", a->cert->crl);
-		if (a->parent != NULL && a->parent->cert != NULL &&
-		    a->parent->cert->mft != NULL)
+		if (a->issuer != NULL && a->issuer->cert != NULL &&
+		    a->issuer->cert->mft != NULL)
 			printf("                          %s\n",
-			    a->parent->cert->mft);
+			    a->issuer->cert->mft);
 		if (a->cert->aia != NULL)
 			printf("                          %s\n", a->cert->aia);
 	}
@@ -299,7 +349,7 @@ proc_parser_file(char *file, unsigned char *buf, size_t len)
 	struct spl *spl = NULL;
 	struct tak *tak = NULL;
 	struct tal *tal = NULL;
-	char *aia = NULL, *aki = NULL;
+	char *aia = NULL;
 	char *crl_uri = NULL;
 	time_t *expires = NULL, *notafter = NULL;
 	struct auth *a;
@@ -318,8 +368,8 @@ proc_parser_file(char *file, unsigned char *buf, size_t len)
 			printf("--\n");
 	}
 
-	if (strncmp(file, "rsync://", strlen("rsync://")) == 0) {
-		file += strlen("rsync://");
+	if (strncmp(file, RSYNC_PROTO, RSYNC_PROTO_LEN) == 0) {
+		file += RSYNC_PROTO_LEN;
 		buf = load_file(file, &len);
 		if (buf == NULL) {
 			warn("parse file %s", file);
@@ -351,7 +401,6 @@ proc_parser_file(char *file, unsigned char *buf, size_t len)
 		if (aspa == NULL)
 			break;
 		aia = aspa->aia;
-		aki = aspa->aki;
 		expires = &aspa->expires;
 		notafter = &aspa->notafter;
 		break;
@@ -359,13 +408,12 @@ proc_parser_file(char *file, unsigned char *buf, size_t len)
 		cert = cert_parse_pre(file, buf, len);
 		if (cert == NULL)
 			break;
-		is_ta = X509_get_extension_flags(cert->x509) & EXFLAG_SS;
+		is_ta = (cert->purpose == CERT_PURPOSE_TA);
 		if (!is_ta)
 			cert = cert_parse(file, cert);
 		if (cert == NULL)
 			break;
 		aia = cert->aia;
-		aki = cert->aki;
 		x509 = cert->x509;
 		if (X509_up_ref(x509) == 0)
 			errx(1, "%s: X509_up_ref failed", __func__);
@@ -383,7 +431,6 @@ proc_parser_file(char *file, unsigned char *buf, size_t len)
 		if (mft == NULL)
 			break;
 		aia = mft->aia;
-		aki = mft->aki;
 		expires = &mft->expires;
 		notafter = &mft->nextupdate;
 		break;
@@ -392,7 +439,6 @@ proc_parser_file(char *file, unsigned char *buf, size_t len)
 		if (gbr == NULL)
 			break;
 		aia = gbr->aia;
-		aki = gbr->aki;
 		expires = &gbr->expires;
 		notafter = &gbr->notafter;
 		break;
@@ -401,7 +447,6 @@ proc_parser_file(char *file, unsigned char *buf, size_t len)
 		if (geofeed == NULL)
 			break;
 		aia = geofeed->aia;
-		aki = geofeed->aki;
 		expires = &geofeed->expires;
 		notafter = &geofeed->notafter;
 		break;
@@ -410,7 +455,6 @@ proc_parser_file(char *file, unsigned char *buf, size_t len)
 		if (roa == NULL)
 			break;
 		aia = roa->aia;
-		aki = roa->aki;
 		expires = &roa->expires;
 		notafter = &roa->notafter;
 		break;
@@ -419,7 +463,6 @@ proc_parser_file(char *file, unsigned char *buf, size_t len)
 		if (rsc == NULL)
 			break;
 		aia = rsc->aia;
-		aki = rsc->aki;
 		expires = &rsc->expires;
 		notafter = &rsc->notafter;
 		break;
@@ -428,7 +471,6 @@ proc_parser_file(char *file, unsigned char *buf, size_t len)
 		if (spl == NULL)
 			break;
 		aia = spl->aia;
-		aki = spl->aki;
 		expires = &spl->expires;
 		notafter = &spl->notafter;
 		break;
@@ -437,7 +479,6 @@ proc_parser_file(char *file, unsigned char *buf, size_t len)
 		if (tak == NULL)
 			break;
 		aia = tak->aia;
-		aki = tak->aki;
 		expires = &tak->expires;
 		notafter = &tak->notafter;
 		break;
@@ -455,9 +496,7 @@ proc_parser_file(char *file, unsigned char *buf, size_t len)
 	if (aia != NULL) {
 		x509_get_crl(x509, file, &crl_uri);
 		parse_load_crl(crl_uri);
-		if (auth_find(&auths, aki) == NULL)
-			parse_load_certchain(aia);
-		a = auth_find(&auths, aki);
+		a = parse_load_certchain(aia);
 		c = crl_get(&crlt, a);
 
 		if ((status = valid_x509(file, ctx, x509, a, c, &errstr))) {
@@ -492,9 +531,15 @@ proc_parser_file(char *file, unsigned char *buf, size_t len)
 			constraints_validate(file, cert);
 		}
 	} else if (is_ta) {
+		expires = NULL;
+		notafter = NULL;
 		if ((tal = find_tal(cert)) != NULL) {
 			cert = ta_parse(file, cert, tal->pkey, tal->pkeysz);
 			status = (cert != NULL);
+			if (status) {
+				expires = &cert->expires;
+				notafter = &cert->notafter;
+			}
 			if (outformats & FORMAT_JSON)
 				json_do_string("tal", tal->descr);
 			else
@@ -504,7 +549,6 @@ proc_parser_file(char *file, unsigned char *buf, size_t len)
 		} else {
 			cert_free(cert);
 			cert = NULL;
-			expires = NULL;
 			status = 0;
 		}
 	}
