@@ -1,4 +1,4 @@
-/*	$OpenBSD: parser.c,v 1.130 2024/03/01 08:10:09 tb Exp $ */
+/*	$OpenBSD: parser.c,v 1.141 2024/06/12 10:03:09 tb Exp $ */
 /*
  * Copyright (c) 2019 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -38,9 +38,7 @@
 
 #include "extern.h"
 
-extern int noop;
-extern int experimental;
-extern int verbose;
+extern int certid;
 
 static X509_STORE_CTX	*ctx;
 static struct auth_tree	 auths = RB_INITIALIZER(&auths);
@@ -90,6 +88,41 @@ repo_add(unsigned int id, char *path, char *validpath)
 		errx(1, "repository already added: id %d, %s", id, path);
 }
 
+/*
+ * Return the issuer by its certificate id, or NULL on failure.
+ * Make sure the AKI is the same as the AKI listed on the Manifest,
+ * and that the SKI of the cert matches with the AKI.
+ */
+static struct auth *
+find_issuer(const char *fn, int id, const char *aki, const char *mftaki)
+{
+	struct auth *a;
+
+	a = auth_find(&auths, id);
+	if (a == NULL) {
+		if (certid <= CERTID_MAX)
+			warnx("%s: RFC 6487: unknown cert with SKI %s", fn,
+			    aki);
+		return NULL;
+	}
+
+	if (mftaki != NULL) {
+		if (strcmp(aki, mftaki) != 0) {
+			warnx("%s: AKI %s doesn't match Manifest AKI %s", fn,
+			    aki, mftaki);
+			return NULL;
+		}
+	}
+
+	if (strcmp(aki, a->cert->ski) != 0) {
+		warnx("%s: AKI %s doesn't match issuer SKI %s", fn,
+		    aki, a->cert->ski);
+		return NULL;
+	}
+
+	return a;
+}
+
 /*
  * Build access path to file based on repoid, path, location and file values.
  */
@@ -141,7 +174,12 @@ proc_parser_roa(char *file, const unsigned char *der, size_t len,
 	if ((roa = roa_parse(&x509, file, entp->talid, der, len)) == NULL)
 		return NULL;
 
-	a = valid_ski_aki(file, &auths, roa->ski, roa->aki, entp->mftaki);
+	a = find_issuer(file, entp->certid, roa->aki, entp->mftaki);
+	if (a == NULL) {
+		X509_free(x509);
+		roa_free(roa);
+		return NULL;
+	}
 	crl = crl_get(&crlt, a);
 
 	if (!valid_x509(file, ctx, x509, a, crl, &errstr)) {
@@ -176,7 +214,12 @@ proc_parser_spl(char *file, const unsigned char *der, size_t len,
 	if ((spl = spl_parse(&x509, file, entp->talid, der, len)) == NULL)
 		return NULL;
 
-	a = valid_ski_aki(file, &auths, spl->ski, spl->aki, entp->mftaki);
+	a = find_issuer(file, entp->certid, spl->aki, entp->mftaki);
+	if (a == NULL) {
+		X509_free(x509);
+		spl_free(spl);
+		return NULL;
+	}
 	crl = crl_get(&crlt, a);
 
 	if (!valid_x509(file, ctx, x509, a, crl, &errstr)) {
@@ -277,6 +320,9 @@ parse_load_crl_from_mft(struct entity *entp, struct mft *mft, enum location loc,
 		goto out;
 	}
 
+	if ((crl->mftpath = strdup(mft->sia)) == NULL)
+		err(1, NULL);
+
 	*crlfile = fn;
 	free(f);
 
@@ -337,7 +383,9 @@ proc_parser_mft_pre(struct entity *entp, char *file, struct crl **crl,
 	if (*crl == NULL)
 		*crl = parse_load_crl_from_mft(entp, mft, DIR_VALID, crlfile);
 
-	a = valid_ski_aki(file, &auths, mft->ski, mft->aki, NULL);
+	a = find_issuer(file, entp->certid, mft->aki, NULL);
+	if (a == NULL)
+		goto err;
 	if (!valid_x509(file, ctx, x509, a, *crl, errstr))
 		goto err;
 	X509_free(x509);
@@ -345,6 +393,7 @@ proc_parser_mft_pre(struct entity *entp, char *file, struct crl **crl,
 
 	mft->repoid = entp->repoid;
 	mft->talid = a->cert->talid;
+	mft->certid = entp->certid;
 
 	now = get_current_time();
 	/* check that now is not before from */
@@ -461,7 +510,8 @@ proc_parser_mft(struct entity *entp, struct mft **mp, char **crlfile,
 				err2 = err1;
 			if (err2 == NULL)
 				err2 = "no valid manifest available";
-			warnx("%s: %s", file2, err2);
+			if (certid <= CERTID_MAX)
+				warnx("%s: %s", file2, err2);
 		}
 
 		mft_free(mft1);
@@ -477,13 +527,11 @@ proc_parser_mft(struct entity *entp, struct mft **mp, char **crlfile,
 
 	if (*mp != NULL) {
 		*crlmtime = crl->thisupdate;
-		if (!crl_insert(&crlt, crl)) {
-			warnx("%s: duplicate AKI %s", file, crl->aki);
-			crl_free(crl);
-		}
-	} else {
-		crl_free(crl);
+		if (crl_insert(&crlt, crl))
+			crl = NULL;
 	}
+	crl_free(crl);
+
 	return file;
 }
 
@@ -496,7 +544,7 @@ proc_parser_mft(struct entity *entp, struct mft **mp, char **crlfile,
  */
 static struct cert *
 proc_parser_cert(char *file, const unsigned char *der, size_t len,
-    const char *mftaki)
+    const struct entity *entp)
 {
 	struct cert	*cert;
 	struct crl	*crl;
@@ -510,7 +558,11 @@ proc_parser_cert(char *file, const unsigned char *der, size_t len,
 	if (cert == NULL)
 		return NULL;
 
-	a = valid_ski_aki(file, &auths, cert->ski, cert->aki, mftaki);
+	a = find_issuer(file, entp->certid, cert->aki, entp->mftaki);
+	if (a == NULL) {
+		cert_free(cert);
+		return NULL;
+	}
 	crl = crl_get(&crlt, a);
 
 	if (!valid_x509(file, ctx, cert->x509, a, crl, &errstr) ||
@@ -534,47 +586,106 @@ proc_parser_cert(char *file, const unsigned char *der, size_t len,
 	 * Add validated CA certs to the RPKI auth tree.
 	 */
 	if (cert->purpose == CERT_PURPOSE_CA)
-		auth_insert(&auths, cert, a);
+		auth_insert(file, &auths, cert, a);
 
 	return cert;
 }
 
-/*
- * Root certificates come from TALs (has a pkey and is self-signed).
- * Parse the certificate, ensure that its public key matches the
- * known public key from the TAL, and then validate the RPKI
- * content.
- *
- * This returns a certificate (which must not be freed) or NULL on
- * parse failure.
- */
-static struct cert *
-proc_parser_root_cert(char *file, const unsigned char *der, size_t len,
-    unsigned char *pkey, size_t pkeysz, int talid)
+static int
+proc_parser_ta_cmp(const struct cert *cert1, const struct cert *cert2)
 {
-	struct cert		*cert;
-
-	/* Extract certificate data. */
-
-	cert = cert_parse_pre(file, der, len);
-	cert = ta_parse(file, cert, pkey, pkeysz);
-	if (cert == NULL)
-		return NULL;
-
-	if (!valid_ta(file, &auths, cert)) {
-		warnx("%s: certificate not a valid ta", file);
-		cert_free(cert);
-		return NULL;
-	}
-
-	cert->talid = talid;
+	if (cert1 == NULL)
+		return -1;
+	if (cert2 == NULL)
+		return 1;
 
 	/*
-	 * Add valid roots to the RPKI auth tree.
+	 * The standards don't specify tiebreakers. While RFC 6487 and other
+	 * sources advise against backdating, it's explicitly allowed and some
+	 * TAs do. Some TAs have also re-issued with new dates and old
+	 * serialNumber.
+	 * Our tiebreaker logic: a more recent notBefore is taken to mean a
+	 * more recent issuance, and thus preferable. Given equal notBefore
+	 * values, prefer the TA cert with the narrower validity window. This
+	 * hopefully encourages TA operators to reduce egregiously long TA
+	 * validity periods.
 	 */
-	auth_insert(&auths, cert, NULL);
 
-	return cert;
+	if (cert1->notbefore < cert2->notbefore)
+		return -1;
+	if (cert1->notbefore > cert2->notbefore)
+		return 1;
+
+	if (cert1->notafter > cert2->notafter)
+		return -1;
+	if (cert1->notafter < cert2->notafter)
+		return 1;
+
+	/*
+	 * Both certs are valid from our perspective. If anything changed,
+	 * prefer the freshly-fetched one. We rely on cert_parse_pre() having
+	 * cached the extensions and thus libcrypto has already computed the
+	 * certs' hashes (SHA-1 for OpenSSL, SHA-512 for LibreSSL). The below
+	 * compares them.
+	 */
+
+	return X509_cmp(cert1->x509, cert2->x509) != 0;
+}
+
+/*
+ * Root certificates come from TALs. Inspect and validate both options and
+ * compare the two. The cert in out_cert must not be freed. Returns the file
+ * name of the chosen TA.
+ */
+static char *
+proc_parser_root_cert(struct entity *entp, struct cert **out_cert)
+{
+	struct cert		*cert1 = NULL, *cert2 = NULL;
+	char			*file1 = NULL, *file2 = NULL;
+	unsigned char		*der = NULL, *pkey = entp->data;
+	size_t			 der_len = 0, pkeysz = entp->datasz;
+	int			 cmp;
+
+	*out_cert = NULL;
+
+	file2 = parse_filepath(entp->repoid, entp->path, entp->file, DIR_VALID);
+	der = load_file(file2, &der_len);
+	cert2 = cert_parse_pre(file2, der, der_len);
+	free(der);
+	cert2 = ta_parse(file2, cert2, pkey, pkeysz);
+
+	if (!noop) {
+		file1 = parse_filepath(entp->repoid, entp->path, entp->file,
+		    DIR_TEMP);
+		der = load_file(file1, &der_len);
+		cert1 = cert_parse_pre(file1, der, der_len);
+		free(der);
+		cert1 = ta_parse(file1, cert1, pkey, pkeysz);
+	}
+
+	if ((cmp = proc_parser_ta_cmp(cert1, cert2)) > 0) {
+		cert_free(cert2);
+		free(file2);
+
+		cert1->talid = entp->talid;
+		auth_insert(file1, &auths, cert1, NULL);
+
+		*out_cert = cert1;
+		return file1;
+	} else {
+		if (cmp < 0 && cert1 != NULL && cert2 != NULL)
+			warnx("%s: cached TA is newer", entp->file);
+		cert_free(cert1);
+		free(file1);
+
+		if (cert2 != 0) {
+			cert2->talid = entp->talid;
+			auth_insert(file2, &auths, cert2, NULL);
+		}
+
+		*out_cert = cert2;
+		return file2;
+	}
 }
 
 /*
@@ -593,10 +704,14 @@ proc_parser_gbr(char *file, const unsigned char *der, size_t len,
 	if ((gbr = gbr_parse(&x509, file, entp->talid, der, len)) == NULL)
 		return NULL;
 
-	a = valid_ski_aki(file, &auths, gbr->ski, gbr->aki, entp->mftaki);
+	a = find_issuer(file, entp->certid, gbr->aki, entp->mftaki);
+	if (a == NULL) {
+		X509_free(x509);
+		gbr_free(gbr);
+		return NULL;
+	}
 	crl = crl_get(&crlt, a);
 
-	/* return value can be ignored since nothing happens here */
 	if (!valid_x509(file, ctx, x509, a, crl, &errstr)) {
 		warnx("%s: %s", file, errstr);
 		X509_free(x509);
@@ -626,7 +741,12 @@ proc_parser_aspa(char *file, const unsigned char *der, size_t len,
 	if ((aspa = aspa_parse(&x509, file, entp->talid, der, len)) == NULL)
 		return NULL;
 
-	a = valid_ski_aki(file, &auths, aspa->ski, aspa->aki, entp->mftaki);
+	a = find_issuer(file, entp->certid, aspa->aki, entp->mftaki);
+	if (a == NULL) {
+		X509_free(x509);
+		aspa_free(aspa);
+		return NULL;
+	}
 	crl = crl_get(&crlt, a);
 
 	if (!valid_x509(file, ctx, x509, a, crl, &errstr)) {
@@ -661,7 +781,9 @@ proc_parser_tak(char *file, const unsigned char *der, size_t len,
 	if ((tak = tak_parse(&x509, file, entp->talid, der, len)) == NULL)
 		return NULL;
 
-	a = valid_ski_aki(file, &auths, tak->ski, tak->aki, entp->mftaki);
+	a = find_issuer(file, entp->certid, tak->aki, entp->mftaki);
+	if (a == NULL)
+		goto out;
 	crl = crl_get(&crlt, a);
 
 	if (!valid_x509(file, ctx, x509, a, crl, &errstr)) {
@@ -670,7 +792,7 @@ proc_parser_tak(char *file, const unsigned char *der, size_t len,
 	}
 
 	/* TAK EE must be signed by self-signed CA */
-	if (a->parent != NULL)
+	if (a->issuer != NULL)
 		goto out;
 
 	tak->talid = a->cert->talid;
@@ -760,15 +882,13 @@ parse_entity(struct entityq *q, struct msgbuf *msgq)
 			tal_free(tal);
 			break;
 		case RTYPE_CER:
-			file = parse_load_file(entp, &f, &flen);
+			if (entp->data != NULL) {
+				file = proc_parser_root_cert(entp, &cert);
+			} else {
+				file = parse_load_file(entp, &f, &flen);
+				cert = proc_parser_cert(file, f, flen, entp);
+			}
 			io_str_buffer(b, file);
-			if (entp->data != NULL)
-				cert = proc_parser_root_cert(file,
-				    f, flen, entp->data, entp->datasz,
-				    entp->talid);
-			else
-				cert = proc_parser_cert(file, f, flen,
-				    entp->mftaki);
 			if (cert != NULL)
 				mtime = cert->notbefore;
 			io_simple_buffer(b, &mtime, sizeof(mtime));
@@ -989,5 +1109,8 @@ proc_parser(int fd)
 
 	ibuf_free(inbuf);
 
+	if (certid > CERTID_MAX)
+		errx(1, "processing incomplete: too many certificates");
+
 	exit(0);
 }
